;****************************************************************************
;****************************************************************************
;***           S P I    R U T I N S 
;***
;****************************************************************************
;****************************************************************************
;*
;* FUNCTION
;*	init_spi
;*
;* DESCRIPTION
;*	Initialize our port pins for use as SPI master.
;*
;* CODE SIZE:
;*	8 words
;*
;***************************************************************************

init_spi:
	AT_inactive		;set latch bit hi (inactive)
	sbi	ddrb,ATss	;make it an output
	;
	sck_lo			;set clk line lo
	sbi	ddrb,sck	;make it an output
	;
	mosi_lo			;set data-out lo
	sbi	ddrb,mosi	;make it an output
	;
	cbi	ddrb,miso	;make it an input
	ret

;***************************************************************************
;*
;* FUNCTION
;*	ena_AT
;*
;* DESCRIPTION
;*	Init data & clock lines, then assert /CS for AT45Dxxx.
;*
;* CODE SIZE:
;*	4 words
;*
;***************************************************************************

ena_AT:
	sck_lo			;(should already be there...)
	mosi_lo
	AT_active
	ret

;***************************************************************************
;*
;* FUNCTION
;*	disa_AT
;*
;* DESCRIPTION
;*	De-assert /CS for AT45Dxxx.  Since this routine is so short,
;*	it might be better to use the SS_INACTIVE statement directly
;*	in higher level code.
;*	Again, if multiple slaves exist, additional copies of this could
;*	be created; or ONE routine that disabled ALL /ss signals could be
;*	used instead to make the code less error-prone due to calling the
;*	wrong Disable routine.
;*
;* CODE SIZE:
;*	2 words
;*
;***************************************************************************

disa_AT:
	AT_inactive
	ret


;***************************************************************************
;*
;* FUNCTION
;*	rw_spi
;*
;* DESCRIPTION
;*	Write a data out on SPI while simultaneously reading in a data.
;*	Data is sent MSB-first, and info read from SPI goes into
;*	the same buffer that the write data is going out from.
;*	Make sure data, clock and /SS are init'd before coming here.
;*	SCK high time is ((delay * 3) + 1) AVR clock cycles.
;*
;* CODE SIZE:
;*	20 words
;* NUMBER OF CYCLES:
;*	Overhead = 8, loop = 16 * (16 + (2* (delay_value*3)))
;	(With call + return + delay=4, it is about 648 cycles.)
;*
;***************************************************************************

rw_spi:	
       ldi	temp,8		;8-bit r/w
	;
spi_loop:
	lsl	spi_lo		;move 0 into D0, all other bits UP one slot,
				; and C ends up being first bit to be sent.
	;
	brcc	lo_mosi
	mosi_hi
	rjmp	mosi_done	;this branch creates setup time on MOSI
lo_mosi:
	mosi_lo
	nop			;also create setup time on MOSI
mosi_done:
	;
	sck_hi
	;
 ;must now time the hi pulse - not much else we can do here but waste time
	;
	set_delay temp,4	;(4 * 3) cycle delay; range is from 1 to 7!
time_hi:
	inc_delay temp		;inc upper nibble until it rolls over; then,
	brcs	time_hi		; C gets CLEARED, & temp has original value
	;
	sck_lo			;drop clock line low
	;
 ;must now delay before reading in SPI data on MISO
	;
	set_delay temp,4
time_lo:
	inc_delay temp
	brcs	time_lo
	;
	sbic	pinb,miso	;after delay, read in SPI bit & put into D0
	inc	spi_lo		;we FORCED D0=0, so use INC to set D0.
	;
	dec	temp
	brne	spi_loop
	ret


;*****************************************************************************
;**  AT45Dxxx Buffer Read
;**
;** ATComm > 	54h Buffer 1 Read
;**		56h Buffer 2 Read
;** ATPoint >	Pointer to Buffer (8 bit)
;*
;*****************************************************************************
ATBuffR:
	push	XL
	push	XH
	mov	spi_lo,ATComm	;Mov Command byte
	rcall	ena_AT		;activate AT /CS
	rcall	rw_spi		;send Command byte
	eor	spi_lo,spi_lo	;spi_lo = 0
	rcall	rw_spi		;send 8 don't care bits
	eor	spi_lo,spi_lo	;spi_lo = 0
	rcall	rw_spi		;send 7 don't care bits & BFA8=0
	mov	spi_lo,ATPoint	;Mow Buffer Pointer
	rcall	rw_spi		;send BFA0..BFA7 
	eor	spi_lo,spi_lo	;spi_lo = 0
	rcall	rw_spi		;send 8 don't care bits
	
	ldi	R17,BFSeg		;32 byte read
	ldi	XL,low(IOBuff)	
	ldi	XH,high(IOBuff)	;	

ATl0:	eor	spi_lo,spi_lo	;spi_lo = 0
	rcall	rw_spi		;receive data 
	st	X+,spi_lo	;store data
	dec	R17
	brne	ATl0
	
	rcall	disa_AT		;deactivate AT /CS
	pop	XH
	pop	XL
	ret


;*****************************************************************************
;**  AT45Dxxx Buffer Write
;**
;** ATComm > 	84h Buffer 1 Write
;**		87h Buffer 2 Write
;** ATPoint >	Pointer to Buffer (8 bit)
;*
;*****************************************************************************
ATBuffW:
	push	XL
	push	XH
	mov	spi_lo,ATComm	;Mov Command byte
	rcall	ena_AT		;activate AT /CS
	rcall	rw_spi		;send Command byte
	eor	spi_lo,spi_lo	;spi_lo = 0
	rcall	rw_spi		;send 8 don't care bits
	eor	spi_lo,spi_lo	;spi_lo = 0
	rcall	rw_spi		;send 7 don't care bits & BFA8=0
	mov	spi_lo,ATPoint	;Mow Buffer Pointer
	rcall	rw_spi		;send BFA0..BFA7 
	
	ldi	R17,BFSeg		;32 byte read
	ldi	XL,low(IOBuff)	
	ldi	XH,high(IOBuff)	;	

ATl1:	ld	spi_lo,X+ 	;spi_lo = data
	rcall	rw_spi		;send data 
	dec	R17
	brne	ATl1
	
	rcall	disa_AT		;deactivate AT /CS
	pop	XH
	pop	XL
	ret



;*****************************************************************************
;**  AT45Dxxx Page/Buffer R/W
;**
;** ATComm > 	0x53 Main Memory Page to Buffer 1 Transfer  
;**		0x55 Main Memory Page to Buffer 2 Transfer 
;**		0x60 Main Memory Page to Buffer 1 Compare		
;**             0x61 Main Memory Page to Buffer 2 Compare
;**             0x83 Buffer 1 to Main Memory Page Program with Built-in Erase
;**             0x86 Buffer 2 to Main Memory Page Program with Built-in Erase
;**             0x88 Buffer 1 to Main Memory Page Program without Built-in Erase
;**             0x89 Buffer 2 to Main Memory Page Program without Built-in Erase
;**             0x58 Auto Page Rewrite Through Buffer 1
;**             0x59 Auto Page Rewrite Through Buffer 2
;**
;** ATPageL >	Page address 11 bit
;*  ATPageH
;*****************************************************************************
ATPageRW:
	mov	spi_lo,ATComm	;Mov Command byte
	rcall	ena_AT		;activate AT /CS
	rcall	rw_spi		;send Command byte
        lsl	ATPageL
	rol	ATPageH
        mov	spi_lo,ATPageH	;Page Address Hi 
	rcall	rw_spi		;send PA7..PA10
	mov	spi_lo,ATPageL
	rcall	rw_spi		;send PA0..PA6 
	eor	spi_lo,spi_lo	;spi_lo = 0
	rcall	rw_spi		;send 8 don't care bits
	rcall	disa_AT		;deactivate AT /CS
	ret

;*****************************************************************************
;**  AT45Dxxx Status 
;**
;** ATComm > 	0x57 Get Status (spi_lo) 
;**
;**   Status:RDY/BUSY, COMP, 0, 1, 1, x, x, x      011 - 4 Mbit device
;*****************************************************************************
ATStatus:
	mov	spi_lo,ATComm	;Mov Command byte
	rcall	ena_AT		;activate AT /CS
	rcall	rw_spi		;send Command byte
	rcall	rw_spi		;get Status in spi_lo
	rcall	disa_AT		;deactivate AT /CS
	ret

